#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/io.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <linux/fs.h>
#include <sys/time.h>
#include <linux/fb.h>

#include "mv_phapi.h"

#define MAX_WIDTH 768
#define MAX_HEIGHT 576

#define PAGE_SIZE       4096
#define PAGE_MASK       (~(PAGE_SIZE-1))


int fd = -1;
int frameflag = 0;
static int FBHandle = -1;
static int Width, Height, LineLen, FrameSize;
__u32  VGAAdd, VGAdepth, VGABytesPerLine;

int img_width, img_height, bpp=3 ;

char * buffer;
void * fb_mem;

int flag;

unsigned long    screensize;
unsigned long fb_mem_offset; 

int Frameplaystop = 0;
pthread_t thrdvga;
pthread_mutex_t stop_frame_mutex;


struct VGA_INFO
{
    __u32 VGA_Add;
    __u32 VGAWidth;
    __u32 VGAHeight;
    __u32 VGAdepth;
    __u32 VGAbytesperline;
};

struct VGA_INFO vi;

static void Error(const char *Msg)
{
        fprintf(stderr, "%s\n", Msg);
        exit(1);
}

void FBOpen(void)
{
   	struct fb_fix_screeninfo fb_fix;
   	struct fb_var_screeninfo fb_var;
	
   	int PageSize = getpagesize();

   	FBHandle = open("/dev/fb0", O_RDWR);
	if ( FBHandle < 0 ) {
		printf("Cann't open file /dev/fb0\n");
		exit(1);
	}
   	if (ioctl(FBHandle, FBIOGET_FSCREENINFO, &fb_fix) == -1 || ioctl(FBHandle, FBIOGET_VSCREENINFO, &fb_var) == -1)
       		Error("Cannot get Frame Buffer information");

   	printf("mem start 0x%lx\n",fb_fix.smem_start);
//    	Width  = fb_var.xres;
//    	Height = fb_var.yres;

	fb_mem_offset = (unsigned long)(fb_fix.smem_start) & (~PAGE_MASK);
// 	screensize = fb_mem_offset + fb_fix.smem_len;
	screensize = fb_var.xres * fb_var.yres * fb_var.bits_per_pixel/8;  
	fb_mem = mmap(NULL, screensize, PROT_READ | PROT_WRITE, MAP_SHARED, FBHandle, 0);
	memset(fb_mem,0,screensize);
   	VGAAdd = fb_fix.smem_start;
	
   	printf("fb_var.bits_per_pixel := %d\n",fb_var.bits_per_pixel);

   	LineLen = fb_fix.line_length;

}

void FBClose(void)
{
   	close(FBHandle);
}


void callback( PVOID pData, PMV_IMAGEINFO pImageInfo, PVOID pUserData )
{
	if(pData != NULL && buffer != NULL)
	{
		memcpy(buffer,(char *)pData,img_width*img_height*bpp);
		flag = 1;
	}
}

void MV_Frameplay()
{
	int stopFlag;
	int i, j;
	__u8 *psrc;
	__u16* pdsc;
	do{
	        psrc= (__u8*)buffer;
	        pdsc = (__u16*)fb_mem;
		if(flag == 1)
		{	flag = 0;
			for(i=0; i<img_height; i++){
				memcpy(pdsc, psrc, img_width*3);
				psrc+=img_width*bpp;
				pdsc+=LineLen/2;
			}
		}
		pthread_mutex_lock(&stop_frame_mutex);
		stopFlag = Frameplaystop;
		pthread_mutex_unlock(&stop_frame_mutex);
	}while(stopFlag == 0);
	pthread_detach(thrdvga);
}

main()
{
	int bright, saturation, hue, contrast;
	int format;
	int index;
	int field;
	int ch;
	RUNOPER status;
	BOOL fb_flag;
	int Frameplay_flag = 0;
	
	

	buffer = (char *)malloc(MAX_WIDTH*MAX_HEIGHT*4);

	FBOpen();

	while(1)
	{
		 
		ch=getchar();
// 		printf("int := %d  ch := %c \n",ch,ch);
		if(ch== 'q' || ch == 'Q')
		{
			if(fd > 0)
    			{
				if( Frameplay_flag == 1 )
				{
					pthread_mutex_lock(&stop_frame_mutex);
					Frameplaystop = 1;
					pthread_mutex_unlock(&stop_frame_mutex);
					pthread_join(thrdvga, NULL);
					Frameplay_flag = 0;
				}

				if( MV_OperateDevice(fd, MVQUERYSTATU) == MVRUN || MV_OperateDevice(fd, MVQUERYSTATU) == MVQUERYSTATU )
				{
					MV_OperateDevice(fd, MVSTOP);
				}	
				if(fd > 0)
    					MV_CloseDevice(fd);
    			}
			if(buffer != NULL)
			{
				free(buffer);
				buffer = NULL;
			}						
			munmap(fb_mem, screensize);
			fb_mem = NULL;
			FBClose();	
			printf("free...........................\n");		
			break;
		}
		 switch(ch)
		{
		case 'h':          /*help information */
		case 'H':
			printf("o/O -- open device which is decided index\n");
			printf("k/K -- start to catch the video\n");
			printf("t/T -- stop to catch the video\n");
			printf("a/A -- start to copy the data to VGA\n");
			printf("z/Z -- stop to copy the data to VGA\n");
			printf("g/G -- save a frame\n");
			printf("q/Q -- quit \n");
			printf("i/I -- set catch field\n");
			printf("r/R -- set catch frame\n");
			printf("b/B -- set bright\n");
			printf("s/S -- set saturation\n");
			printf("c/C -- set contrast\n");
			printf("d/D -- set hue\n");
			printf("f/F -- set color format\n");
			printf("w/W -- set the output image width\n");
			printf("e/E -- set the output image height\n");
	        	break;
		case 'g':
		case 'G':
			MV_WriteBmp("1.bmp", buffer, img_width, img_height, bpp);
			break;
		case 'k':
		case 'K':
			if(fd <= 0)
				break;
			status = MVRUN;
			MV_OperateDevice(fd,MVRUN);
			break;
		case 't':
		case 'T':
			if(fd <= 0)
				break;
			status = MVSTOP;
			MV_OperateDevice(fd,MVSTOP);
			break;
		case 'a':
		case 'A':
			if(fd <= 0)
				break;
			Frameplaystop = 0;
			pthread_create(&thrdvga , NULL, (void *)MV_Frameplay, NULL);
			Frameplay_flag = 1;
			break;
		case 'z':
		case 'Z':
			if(fd <= 0)
				break;
			pthread_mutex_lock(&stop_frame_mutex);
			Frameplaystop = 1;
			pthread_mutex_unlock(&stop_frame_mutex);
			if(Frameplay_flag == 1)
			{
				pthread_join(thrdvga, NULL);
				Frameplay_flag = 0;
			}
			break;
		case 'i':
		case 'I':
			if(fd <= 0)
				break;
	evenorodd:	printf("FIELD_TOP--1, FIELD_BOTTOM--2\n");
			scanf("%d",&field);
			if(field == 1 || field ==2)
			{
				img_height = img_height/2;
				MV_SetDeviceParameter(fd,WORK_FIELD,field);
			}
			else
				goto evenorodd;
			break;
		case 'r':
		case 'R':
			if(fd <= 0)
				break;
			MV_SetDeviceParameter(fd,WORK_FIELD,0);
			frameflag = 1;
			break;
		case 'b':         /* set bright */
		case 'B':
			if(fd <= 0)
				break;
			if(status == MVRUN)
				MV_OperateDevice(fd,MVSTOP);
			printf("please input bright : ");
			scanf("%d",&bright);
			MV_SetDeviceParameter(fd,ADJUST_LUMINANCE,bright);
			if(status == MVRUN)
				MV_OperateDevice(fd,MVRUN);
	        	break;
		case 's':          /* set saturation */
		case 'S':
			if(fd <= 0)
				break;
			if(status == MVRUN)
				MV_OperateDevice(fd,MVSTOP);
			printf("please input saturation : ");
			scanf("%d",&saturation);
			MV_SetDeviceParameter(fd,ADJUST_SATURATION,saturation);
			if(status == MVRUN)
				MV_OperateDevice(fd,MVRUN);
			break;
		case 'c':           /* set contrast */
		case 'C':
			if(fd <= 0)
				break;
			if(status == MVRUN)
				MV_OperateDevice(fd,MVSTOP);
			printf("please input contrast : ");
			scanf("%d",&contrast);
			MV_SetDeviceParameter(fd,ADJUST_CONTRAST,contrast);
			if(status == MVRUN)
				MV_OperateDevice(fd,MVRUN);
	        	break;
		case 'd':           /* set hue */
		case 'D':
			if(fd <= 0)
				break;
			if(status == MVRUN)
				MV_OperateDevice(fd,MVSTOP);
			printf("please input hue : ");
			scanf("%d",&hue);
			MV_SetDeviceParameter(fd,ADJUST_HUE,hue);
			if(status == MVRUN)
				MV_OperateDevice(fd,MVRUN);
			break;
		case 'w':           /* set the output image width */
		case 'W':
			if(fd <= 0)
				break;
			if(status == MVRUN)
				MV_OperateDevice(fd,MVSTOP);
			printf("please input img_width(16--768) : ");
			scanf("%d",&img_width);
			MV_SetDeviceParameter(fd,GRAB_WIDTH,img_width);
			if(status == MVRUN)
				MV_OperateDevice(fd,MVRUN);
			break;
		case 'e':           /* set the output image height */
		case 'E':
			if(fd <= 0)
				break;
			if(status == MVRUN)
				MV_OperateDevice(fd,MVSTOP);
			printf("please input img_width(16--768) : ");
			scanf("%d",&img_height);
			MV_SetDeviceParameter(fd,GRAB_HEIGHT,img_height);
			if(status == MVRUN)
				MV_OperateDevice(fd,MVRUN);
			break;
		case 'o':           /* open device which is decided index */
		case 'O':
			printf("please input device num : ");
			scanf("%d",&index);
			if(index<0 || index>3)
				break;	
			if(fd > 0)
    			{
				if(MV_OperateDevice(fd, MVQUERYSTATU) == MVRUN || MV_OperateDevice(fd, MVQUERYSTATU) == MVPAUSE)
				{
					MV_OperateDevice(fd, MVSTOP);
					status = MVSTOP;
				}	
				MV_CloseDevice(fd);
			}
			fd = MV_OpenDevice(index);
			printf("fd := %d \n",fd);
			MV_SetCallBack(fd, callback, NULL);
			img_width = MV_GetDeviceParameter(fd, GRAB_WIDTH);
			img_height = MV_GetDeviceParameter(fd, GRAB_HEIGHT);
			bpp = MV_GetDeviceParameter(fd, GRAB_BPP);
			break;
		case 'f':           /* set color format */
		case 'F':
			if(fd <= 0)
				break;
			if(status == MVRUN)
			{
				MV_OperateDevice(fd, MVSTOP);
			}
			printf( "	0--RGB24 bpp = 3\n");
			printf( "	1--RGB32 bpp = 4\n");
			printf( "	2--RGB565  bpp = 2\n");
			printf( "	3--RGB1555 bpp = 2\n");
			printf( "	4--RGB5515 bpp = 2\n");
			printf( "	5--RGB8 bpp = 1\n");
			printf("\n");
			printf("please input the color num : ");
			scanf("%d",&format);
			MV_SetDeviceParameter(fd, GARB_BITDESCRIBE, format);
			bpp = MV_GetDeviceParameter(fd, GRAB_BPP);
			if(status == MVRUN)
				MV_OperateDevice(fd, MVRUN);
		default:
			break;
		}	
		ch=getchar();
	}
}

